/*
 * Copyright (C) 2023-2024
 * Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package server

import (
	"fmt"
	"github.com/stretchr/testify/assert"
	"info-manager-proxy/internal/mock"
	"net/http"
	"testing"
)

func TestRegisterHomeEndpoint(t *testing.T) {
	SetUpHttpGateway()

	resp, err := http.Get("http://localhost:8082/")

	assert.NoError(t, err)
	assert.Equal(t, http.StatusOK, resp.StatusCode)
}

func TestRegisterInformationEndpoints(t *testing.T) {
	t.Run("should get information", func(t *testing.T) {
		mock.SetUpGrpcServer()
		SetUpHttpGateway()

		resp, err := http.Get("http://localhost:8082/api/v1/informations/testPostfachId")

		assert.NoError(t, err)
		assert.Equal(t, http.StatusOK, resp.StatusCode)
	})

	t.Run("should get information by id", func(t *testing.T) {
		mock.SetUpGrpcServer()
		SetUpHttpGateway()

		resp, err := http.Get("http://localhost:8082/api/v1/information/testId")

		assert.NoError(t, err)
		assert.Equal(t, http.StatusOK, resp.StatusCode)
	})
}

func TestErrorHandler(t *testing.T) {
	assertErroneousRequest := func(postfachId string, expectedStatusCode int) {
		mock.SetUpGrpcServer()
		SetUpHttpGateway()

		resp, err := http.Get(fmt.Sprintf("http://localhost:8082/api/v1/informations/%v", postfachId))

		assert.NoError(t, err)
		assert.Equal(t, expectedStatusCode, resp.StatusCode)
	}

	t.Run("should have bad request error", func(t *testing.T) {
		assertErroneousRequest("", http.StatusBadRequest)
	})

	t.Run("should have unavailable error", func(t *testing.T) {
		assertErroneousRequest("service_unavailable", http.StatusServiceUnavailable)
	})

	t.Run("should have internal server error", func(t *testing.T) {
		assertErroneousRequest("internal_server_error", http.StatusInternalServerError)
	})
}
