/*
 * Copyright (C) 2023-2024
 * Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package server

import (
	"context"
	"fmt"
	"info-manager-proxy/internal/config"
	"info-manager-proxy/internal/mock"
	"net/http"
    log "github.com/sirupsen/logrus"
	"github.com/grpc-ecosystem/grpc-gateway/v2/runtime"
)

func StartHttpGateway(conf config.Config) *http.Server {
	ctx := context.Background()
	ctx, cancel := context.WithCancel(ctx)
	defer cancel()

	mux := runtime.NewServeMux(runtime.WithErrorHandler(ErrorHandler))
	grpcUrl := conf.Grpc.Url
	if conf.Grpc.Mock {
		grpcUrl = fmt.Sprintf("localhost:%d", mock.GrpcMockPort)
	}

	RegisterHomeEndpoint(mux)
	RegisterInformationEndpoints(ctx, mux, grpcUrl)

	httpServer := &http.Server{
		Addr:    fmt.Sprintf(":%d", conf.Server.Port),
		Handler: RequestLoggingMiddleware(mux),
	}

	log.Info(fmt.Sprintf("HTTP gateway listening on port %d", conf.Server.Port))
	if err := httpServer.ListenAndServe(); err != nil {
		log.Fatal(fmt.Sprintf("HTTP gateway failed to serve: %v", err))
	}

	return httpServer
}
