/*
 * Copyright (C) 2023-2024
 * Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package server

import (
	"bytes"
	"github.com/stretchr/testify/assert"
	"info-manager-proxy/internal/mock"
	"net/http"
	"testing"
	log "github.com/sirupsen/logrus"
)

func TestRequestLoggingMiddleware(t *testing.T) {
	t.Run("should log request", func(t *testing.T) {
		mock.SetUpGrpcServer()
		SetUpHttpGateway()

		var buf bytes.Buffer
		log.SetOutput(&buf)

		originalFlags := log.Baselog.Flags()
		defer func() {
			log.SetOutput(nil)
			log.SetFlags(originalFlags)
		}()

		http.Get("http://localhost:8082/api/v1/information/testId")

		logOutput := buf.String()
		assert.Contains(t, logOutput, "received GET request for /api/v1/information/testId with body")
		assert.Contains(t, logOutput, "successfully handled GET request for /api/v1/information/testId with body")
	})

	t.Run("should not log request", func(t *testing.T) {
		mock.SetUpGrpcServer()
		SetUpHttpGateway()

		var buf bytes.Buffer
		log.Baselog.SetOutput(&buf)

		originalFlags := log.Baselog.Flags()
		defer func() {
			log.Baselog.SetOutput(nil)
			log.Baselog.SetFlags(originalFlags)
		}()

		http.Get("http://localhost:8082/")

		logOutput := buf.String()
		assert.Empty(t, logOutput)
	})
}
